/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | foam-extend: Open Source CFD
   \\    /   O peration     | Version:     4.0
    \\  /    A nd           | Web:         http://www.foam-extend.org
     \\/     M anipulation  | For copyright notice see file Copyright
-------------------------------------------------------------------------------
License
    This file is part of foam-extend.

    foam-extend is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    foam-extend is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with foam-extend.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::solidBodyMotionFunctions::forceBased

Description
    Translation motion function with sine law ramping for velocity.

SourceFiles
    forceBased.C

\*---------------------------------------------------------------------------*/

#ifndef forceBased_H
#define forceBased_H

#include "solidBodyMotionFunction.H"
#include "immersedBoundaryForces.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace solidBodyMotionFunctions
{

/*---------------------------------------------------------------------------*\
                          Class forceBased Declaration
\*---------------------------------------------------------------------------*/

class forceBased
:
    public solidBodyMotionFunction
{
    // Private data

        //- Velocity
        vector g_;
        scalar still_;
        scalar mass_;
        mutable vector velocity_;
        mutable vector oldPosition_;
		mutable scalar currentTime_;

        //- Force Calculation object
        immersedBoundaryForces imBForces_;

	    // Private Member Functions

        //- Disallow copy construct
        forceBased(const forceBased&);

        //- Disallow default bitwise assignment
        void operator=(const forceBased&);

	//- Position calculations
        vector newPosition() const;
	vector newVelocity() const;

public:

		
    //- Runtime type information
    TypeName("forceBased");


    // Constructors

        //- Construct from components
        forceBased
        (
            const dictionary& SBMFCoeffs,
            const Time& runTime
        );


    // Destructor

        virtual ~forceBased();


    // Member Functions
		
        //- Return the solid-body motion transformation septernion
        virtual septernion transformation() const;

        //- Return the solid-body motion velocity
        virtual septernion velocity() const;

        //- Update properties from given dictionary
        virtual bool read(const dictionary& SBMFCoeffs);
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace solidBodyMotionFunctions
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
