/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | foam-extend: Open Source CFD
   \\    /   O peration     | Version:     4.0
    \\  /    A nd           | Web:         http://www.foam-extend.org
     \\/     M anipulation  | For copyright notice see file Copyright
-------------------------------------------------------------------------------
License
    This file is part of foam-extend.

    foam-extend is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    foam-extend is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with foam-extend.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "forceBased.H"
#include "addToRunTimeSelectionTable.H"
#include "mathematicalConstants.H"

using namespace Foam::mathematicalConstant;

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace solidBodyMotionFunctions
{
    defineTypeNameAndDebug(forceBased, 0);
    addToRunTimeSelectionTable
    (
        solidBodyMotionFunction,
        forceBased,
        dictionary
    );
};
};


// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //
Foam::vector
Foam::solidBodyMotionFunctions::forceBased::newPosition() const 
{

	scalar t = time_.value();
    scalar dT = time_.deltaT().value(); 

    if (t < (still_*dT))//+currentTime_))
    {
            return oldPosition_;
    }
    else if (t< 2*(still_*dT))
    {
	    if (t > currentTime_) 
            {
		        currentTime_ = t;
                vector gravityForce = mass_*g_;
                Info << "Gravity: " << g_ << endl;
                Info << "Gravitational Force: " << gravityForce << endl;
                vector pressureForces = imBForces_.calcForcesMoment().first().first();
                vector viscousForces = imBForces_.calcForcesMoment().first().second();
		        vector totalForce = gravityForce+pressureForces+viscousForces;

                Info << "Pressure Force: " << pressureForces << endl;
                Info << "Viscous Force: " << viscousForces << endl;
                Info << "total force: " << totalForce << endl;
                Info << "Delta t: " << dT << endl;		
    	        Info << "old positon: " << oldPosition_ << endl;
                
				vector velocity(0.01,0,0);
                vector dx = velocity*dT;

                Info << "old velocity: " << velocity_ << endl;
                velocity_ = vector(0,0,0);//dx/dT;//velocity_+totalForce/mass_*dT;
                Info << "new velocity: " << velocity_ << endl;

                vector newPos=oldPosition_+dx;
                		
		        Info << "new position: " << newPos << endl;
		        Info << "new old positon: " << oldPosition_ << endl;
		        oldPosition_ = newPos;
		        return oldPosition_;
		    }
	        else 
            {
                return oldPosition_;
            }
    }
    else
    {
	    if (t > currentTime_) 
            {
		        currentTime_ = t;
                vector gravityForce = mass_*g_;
                Info << "Gravity: " << g_ << endl;
                Info << "Gravitational Force: " << gravityForce << endl;
                vector pressureForces = imBForces_.calcForcesMoment().first().first();
                vector viscousForces = imBForces_.calcForcesMoment().first().second();
		        vector totalForce = gravityForce+pressureForces+viscousForces;

                Info << "Pressure Force: " << pressureForces << endl;
                Info << "Viscous Force: " << viscousForces << endl;
                Info << "total force: " << totalForce << endl;
                Info << "Delta t: " << dT << endl;		
    	        Info << "old positon: " << oldPosition_ << endl;
                

                vector dx = velocity_*dT+0.5*totalForce/mass_*pow(dT,2.0);

                Info << "old velocity: " << velocity_ << endl;
                velocity_ = vector(0,0,0);//dx/dT;//velocity_+totalForce/mass_*dT;
                Info << "new velocity: " << velocity_ << endl;

                vector newPos=oldPosition_+dx;
                		
		        Info << "new position: " << newPos << endl;
		        Info << "new old positon: " << oldPosition_ << endl;
		        oldPosition_ = newPos;
		        return oldPosition_;
		    }
	        else 
            {
                return oldPosition_;
            }
    }
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::solidBodyMotionFunctions::forceBased::forceBased
(
    const dictionary& SBMFCoeffs,
    const Time& runTime
)
:
    solidBodyMotionFunction(SBMFCoeffs, runTime),
    g_(0,0,0),
    velocity_(0,0,0),
    oldPosition_(0,0,0),
    currentTime_(0.0),
    imBForces_("test", time_.lookupObject<objectRegistry>(polyMesh::defaultRegion), SBMFCoeffs_, true)

{
    read(SBMFCoeffs);
}


// * * * * * * * * * * * * * * * * Destructors * * * * * * * * * * * * * * * //

Foam::solidBodyMotionFunctions::forceBased::~forceBased()
{}


// * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * * //

Foam::septernion
Foam::solidBodyMotionFunctions::forceBased::transformation() const
{

    septernion TR;

        TR = septernion
        (
            newPosition(),
            quaternion::I
        );

//    Info << "Doing some force stuff: " 
//        << "The force on the object is = " << imBForces_.calcForcesMoment().first()
//        << endl;

//	Info << "Pressure force = " << imBForces_.calcForcesMoment().first().first()
//		<< endl;
    return TR;
}


Foam::septernion
Foam::solidBodyMotionFunctions::forceBased::velocity() const
{
    septernion TV
    (
        velocity_,
        quaternion::I/time_.deltaT().value()
    );

    Info<< "solidBodyMotionFunctions::forceBased::transformation(): "
        << "Time = " << time_.value() << " velocity: " << TV << endl;

    return TV;
}


bool Foam::solidBodyMotionFunctions::forceBased::read
(
    const dictionary& SBMFCoeffs
)
{
    solidBodyMotionFunction::read(SBMFCoeffs);

    SBMFCoeffs_.lookup("gravity") >> g_;
    SBMFCoeffs_.lookup("mass") >> mass_;
    SBMFCoeffs_.lookup("stationary") >> still_;

    return true;
}


// ************************************************************************* //
