#!/usr/bin/python

import os
import sys
import math
import matplotlib.pyplot as plt

forces_file = "forces/0/immersedBoundaryForces.dat"

if not os.path.isfile(forces_file):
    print "Forces file not found at "+forces_file
    print "Be sure that the case has been run and you have the right directory!"
    print "Exiting."
    sys.exit()

def line2dict(line):
    tokens_unprocessed = line.split()
    tokens = [x.replace(")","").replace("(","") for x in tokens_unprocessed]
    floats = [float(x) for x in tokens]
    data_dict = {}
    data_dict['time'] = floats[0]
    force_dict = {}
    force_dict['pressure'] = floats[1:4]
    force_dict['viscous'] = floats[4:7]
    force_dict['porous'] = floats[7:10]
#    moment_dict = {}
#    moment_dict['pressure'] = floats[10:13]
#    moment_dict['viscous'] = floats[13:16]
#    moment_dict['porous'] = floats[16:19]
    data_dict['force'] = force_dict
#    data_dict['moment'] = moment_dict
    return data_dict

time = []
drag = []
lift = []
moment = []
with open(forces_file,"r") as datafile:
    for line in datafile:
        if line[0] == "#":
            continue
        data_dict = line2dict(line)
        time += [data_dict['time']]
        drag += [data_dict['force']['pressure'][0] + data_dict['force']['viscous'][0]]
        lift += [data_dict['force']['pressure'][1] + data_dict['force']['viscous'][1]]
#        moment += [data_dict['moment']['pressure'][2] + data_dict['moment']['viscous'][2]]
datafile.close()
#print(drag)
#print(time)
fig=plt.figure(1)
ax = fig.add_subplot(111)
plt.xlim([0.0,0.45])
plt.ylabel('Force [N]')
plt.xlabel('Time [s]')

font = {'family' : 'normal',
        'weight' : 'bold',
        'size'   : 20}
plt.rc('font', **font)
plt.plot([0.19,0.19],[-8,8],color='r',linewidth='2')
plt.plot([0.39,0.39],[-8,8],color='r',linewidth='2')
plt.plot([0,0.50],[0,0],linestyle='--',color='k')
ax.annotate('Constant velocity', xy=(0.2, 1), xytext=(0.21, 6))
ax.annotate('No motiona allowed', xy=(0.2, 1), xytext=(0.01, -20))
plt.plot(time,drag, color='b', linewidth='2')
plt.ylim([-50,50])
plt.show()
